#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include "iolib.h"
#define PROGNAME "setlock"
#include "freedt.h"

int error = 1;

void fail(const char *msg1, const char *msg2) {
	if (error)
		die2(msg1, msg2);
	else
		exit(0);
}

int main(int argc, char **argv) {
	int fd, block = 0;

	istdinit();

	while (1) {
		/* The + is here so that GNU getopt doesn't attempt
		   to parse options of programs we execute. */
		char c = getopt(argc, argv, "+nNxX");
		if (c == -1)
			break;
		switch (c) {
		case 'n':
			block = LOCK_NB;
			break;
		case 'N':
			block = 0;
			break;
		case 'x':
			error = 0;
			break;
		case 'X':
			error = 1;
			break;
		default:
			die("unknown option");
		}
	}

	if ((argc - optind) < 2)
		die("usage: setlock [options] file command ...");

	fd = open(argv[optind], O_WRONLY | O_CREAT, 0600);
	if (fd < 0)
		fail(argv[optind], "unable to open");
	if (fcntl(fd, F_SETFD, 0) < 0)
		fail(argv[optind], "unable to set fd non-close-on-exec");
	if (flock(fd, LOCK_EX | block) < 0)
		fail(argv[optind], "unable to lock");

	execvp(argv[optind + 1], &argv[optind + 1]);
	die2(argv[optind + 1], "unable to exec");

	return 0; /* NOTREACHED */
}

